package net.npaka.fileex;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

//ファイルの読み書き
public class FileEx extends Activity 
    implements View.OnClickListener {
    private EditText editText;//エディットテキスト
    private Button   btnWrite;//書き込みボタン
    private Button   btnRead; //読み込みボタン
    
    //初期化
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        //レイアウトの生成
        LinearLayout layout=new LinearLayout(this);
        layout.setBackgroundColor(Color.rgb(255,255,255));
        layout.setOrientation(LinearLayout.VERTICAL);
        setContentView(layout);            
        
        //エディットテキストの生成
        editText=new EditText(this);
        editText.setText("",EditText.BufferType.NORMAL);
        setLLParams(editText,240,50);
        layout.addView(editText);

        //書き込みボタンの生成
        btnWrite=new Button(this);
        btnWrite.setText("書き込み");
        btnWrite.setOnClickListener(this);
        setLLParams(btnWrite);  
        layout.addView(btnWrite);        

        //読み込みボタンの生成
        btnRead=new Button(this);
        btnRead.setText("読み込み");
        btnRead.setOnClickListener(this);
        setLLParams(btnRead);  
        layout.addView(btnRead); 
    }

    //ボタンクリックイベントの処理
    public void onClick(View v) {
        if (v==btnWrite) {
            try {
                String str=editText.getText().toString();
                str2file(this,str,"test.txt");
            } catch (Exception e) {
                showDialog(this,"エラー","書き込み失敗しました ");
            }            
        } else  if (v==btnRead) {
            try {
                String str=file2str(this,"test.txt");
                editText.setText(str,TextView.BufferType.EDITABLE);        
            } catch (Exception e) {
                showDialog(this,"エラー","読み込み失敗しました ");
            }            
        }
    }     
    
    //文字列→ファイル
    private static void str2file(Context context,
        String str,String fileName) throws Exception {
        //文字列のバイト配列変換(1)
    	data2file(context,str.getBytes(),fileName);
    }      

    //文字列→バイトデータ
    private static String file2str(Context context,
        String fileName) throws Exception {  
        byte[] w=file2data(context,fileName);
        //バイト配列の文字列変換(2)
        return new String(w);
    }    
    
    //バイトデータ→ファイル
    private static void data2file(Context context,
        byte[] w,String fileName) throws Exception {
        OutputStream out=null;
        try {
            //ファイル出力ストリームのオープン(3)
            out=context.openFileOutput(fileName,
                Context.MODE_PRIVATE);
            
            //バイト配列の書き込み(4)
            out.write(w,0,w.length);
            
            //ファイル出力ストリームのクローズ(5)
            out.close();
        } catch (Exception e) {
            try {
                if (out!=null) out.close();
            } catch (Exception e2) {
            }
            throw e;
        }
    } 
    
    //ファイル→バイトデータ
    private static byte[] file2data(Context context,
        String fileName) throws Exception {
        int size;
        byte[] w=new byte[1024]; 
        InputStream in=null;
        ByteArrayOutputStream out=null;
        try {
            //ファイル入力ストリームのオープン(6)
            in=context.openFileInput(fileName);
            
            //バイト配列の読み込み(7)            
            out=new ByteArrayOutputStream();
            while (true) {
                size=in.read(w);
                if (size<=0) break;
                out.write(w,0,size);
            }
            out.close();
            
            //ファイル入力ストリームのクローズ(8)
            in.close();
            
            //ByteArrayOutputStreamオブジェクトのバイト配列化(9)
            return out.toByteArray();
        } catch (Exception e) {
            try {
                if (in!=null) in.close();
                if (out!=null) out.close();
            } catch (Exception e2) {
            }
            throw e;
        }
    }
    
    //ダイアログの表示
    private static void showDialog(final Activity activity,String title,String text) {
        AlertDialog.Builder ad=new AlertDialog.Builder(activity);
        ad.setTitle(title);
        ad.setMessage(text);
        ad.setPositiveButton("OK",new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog,int whichButton) {
                activity.setResult(Activity.RESULT_OK);
            }
        });
        ad.create();
        ad.show();
    }    
    
    //ライナーレイアウトのパラメータ指定
    private static void setLLParams(View view) {
        view.setLayoutParams(new LinearLayout.LayoutParams(
            LinearLayout.LayoutParams.WRAP_CONTENT,
            LinearLayout.LayoutParams.WRAP_CONTENT));
    }  

    //ライナーレイアウトのパラメータ指定
    private static void setLLParams(View view,int w,int h) {
        view.setLayoutParams(new LinearLayout.LayoutParams(w,h));
    }
}