package net.npaka.sqliteex;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;

//データベースの読み書き
public class SQLiteEx extends Activity 
    implements View.OnClickListener {
    private final static String DB_NAME ="test.db";//DB名
    private final static String DB_TABLE="test";   //テーブル名
    private final static int    DB_VERSION=1;      //バージョン
    
    private EditText       editText;//エディットテキスト
    private Button         btnWrite;//書き込みボタン
    private Button         btnRead; //読み込みボタン
    private SQLiteDatabase db;      //データベースオブジェクト    
    
    //初期化
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        //レイアウトの生成
        LinearLayout layout=new LinearLayout(this);
        layout.setBackgroundColor(Color.rgb(255,255,255));
        layout.setOrientation(LinearLayout.VERTICAL);
        setContentView(layout);              
        
        //エディットテキストの生成
        editText=new EditText(this);
        editText.setText("",EditText.BufferType.NORMAL);
        setLLParams(editText,240,50);
        layout.addView(editText);

        //書き込みボタンの生成
        btnWrite=new Button(this);
        btnWrite.setText("書き込み");
        btnWrite.setOnClickListener(this);
        setLLParams(btnWrite);  
        layout.addView(btnWrite);        

        //読み込みボタンの生成
        btnRead=new Button(this);
        btnRead.setText("読み込み");
        btnRead.setOnClickListener(this);
        setLLParams(btnRead);  
        layout.addView(btnRead);       

        //データベースオブジェクトの取得(5)
        DBHelper dbHelper=new DBHelper(this);
        db=dbHelper.getWritableDatabase();               
    }

    //ボタンクリックイベントの処理
    public void onClick(View v) {
        if (v==btnWrite) {
            try {
                String str=editText.getText().toString();
                writeDB(str);                
            } catch (Exception e) {
                showDialog(this,"エラー","書き込み失敗しました ");
            }
            
        } else if (v==btnRead) {
            try {
                String str=readDB();
                editText.setText(str);               
            } catch (Exception e) {
                showDialog(this,"エラー","読み込み失敗しました ");
            }            
        }
    }    

    //データベースへの書き込み(6)
    private void writeDB(String info) throws Exception {
        ContentValues values=new ContentValues();
        values.put("id","0");
        values.put("info",info);
        int colNum=db.update(DB_TABLE,values,null,null);
        if (colNum==0) db.insert(DB_TABLE,"",values);
    }

    //データベースからの読み込み(7)
    private String readDB() throws Exception {
        Cursor c=db.query(DB_TABLE,new String[]{"id","info"},
            "id='0'",null,null,null,null);
        if (c.getCount()==0) throw new Exception();
        c.moveToFirst();
        return c.getString(1);
    }     
    
    //データベースヘルパーの定義(1)
    private static class DBHelper extends SQLiteOpenHelper {
        //データベースヘルパーのコンストラクタ(2)
        public DBHelper(Context context) {
            super(context,DB_NAME,null,DB_VERSION);
        }
        
        //データベースの生成(3)
        @Override
        public void onCreate(SQLiteDatabase db) {
            db.execSQL(
                "create table if not exists "+
                DB_TABLE+"(id text primary key,info text)");
        }

        //データベースのアップグレード(4)
        @Override
        public void onUpgrade(SQLiteDatabase db,
            int oldVersion,int newVersion) {
            db.execSQL(
                "drop talbe if exists "+DB_TABLE);
            onCreate(db);
        }
    }    
    
    //ダイアログの表示
    private static void showDialog(final Activity activity,String title,String text) {
        AlertDialog.Builder ad=new AlertDialog.Builder(activity);
        ad.setTitle(title);
        ad.setMessage(text);
        ad.setPositiveButton("OK",new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog,int whichButton) {
                activity.setResult(Activity.RESULT_OK);
            }
        });
        ad.create();
        ad.show();
    }    
    
    //ライナーレイアウトのパラメータ指定
    private static void setLLParams(View view) {
        view.setLayoutParams(new LinearLayout.LayoutParams(
            LinearLayout.LayoutParams.WRAP_CONTENT,
            LinearLayout.LayoutParams.WRAP_CONTENT));
    }  

    //ライナーレイアウトのパラメータ指定
    private static void setLLParams(View view,int w,int h) {
        view.setLayoutParams(new LinearLayout.LayoutParams(w,h));
    }
}