package net.npaka.dbproviderex;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.database.Cursor;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;

//DBを提供するコンテンツプロバイダ
public class DBProviderEx extends Activity 
    implements View.OnClickListener {
    private EditText editText;//エディットテキスト
    private Button   btnWrite;//書き込みボタン
    private Button   btnRead; //読み込みボタン
    
    //初期化
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        //レイアウトの生成
        LinearLayout layout=new LinearLayout(this);
        layout.setBackgroundColor(Color.rgb(255,255,255));
        layout.setOrientation(LinearLayout.VERTICAL);
        setContentView(layout);            
        
        //エディットテキストの生成
        editText=new EditText(this);
        editText.setWidth(240);
        editText.setText("",EditText.BufferType.NORMAL);
        setLLParams(editText);
        layout.addView(editText);

        //書き込みボタンの生成
        btnWrite=new Button(this);
        btnWrite.setText("書き込み");
        btnWrite.setOnClickListener(this);
        setLLParams(btnWrite);  
        layout.addView(btnWrite);        

        //読み込みボタンの生成
        btnRead=new Button(this);
        btnRead.setText("読み込み");
        btnRead.setOnClickListener(this);
        setLLParams(btnRead);  
        layout.addView(btnRead);
    }

    //ボタンクリックイベントの処理
    public void onClick(View v) {
        if (v==btnWrite) {
            try {
                writeDB(editText.getText().toString());                
            } catch (Exception e) {
                showDialog(this,"エラー","書き込み失敗しました ");
            }            
        } else if (v==btnRead) {
            try {
                editText.setText(readDB());               
            } catch (Exception e) {
                showDialog(this,"エラー","読み込み失敗しました "+e.toString());
            }            
        }
    }    
    
    //DBへの書き込み
    private void writeDB(String info) throws Exception {
        //コンテントプロバイダが提供するデータベースを示すURI(1)
        Uri uri=Uri.parse("content://net.npaka.dbprovider/"); 
       
        //コンテントプロバイダが提供するデータベースへのアクセス(2)
        ContentValues cv=new ContentValues();
        cv.put("id","0");
        cv.put("info",info);
        this.getContentResolver().update(uri,cv,null,null);
    }

    //DBからの読み込み
    private String readDB() throws Exception {
        //コンテントプロバイダが提供するデータベースを示すURI(1)
        Uri uri=Uri.parse("content://net.npaka.dbprovider/"); 
        
        //コンテントプロバイダが提供するデータベースへのアクセス(2)
        Cursor c=this.getContentResolver().query(uri,
            new String[]{"id","info"},"id='0'",null,null);
        if (c.getCount()==0) throw new Exception();
        c.moveToFirst();
        return c.getString(1);
    }
    
    //ダイアログの表示
    private static void showDialog(final Activity activity,String title,String text) {
        AlertDialog.Builder ad=new AlertDialog.Builder(activity);
        ad.setTitle(title);
        ad.setMessage(text);
        ad.setPositiveButton("OK",new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog,int whichButton) {
                activity.setResult(Activity.RESULT_OK);
            }
        });
        ad.create();
        ad.show();
    }    
    
    //ライナーレイアウトのパラメータ指定
    private static void setLLParams(View view) {
        view.setLayoutParams(new LinearLayout.LayoutParams(
            LinearLayout.LayoutParams.WRAP_CONTENT,
            LinearLayout.LayoutParams.WRAP_CONTENT));
    } 
}