package net.npaka.actiongame;
import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import java.util.Random;

//アクションゲーム
public class ActionView extends SurfaceView 
    implements SurfaceHolder.Callback,Runnable {
    //シーン定数(1)
    private final static int
        S_TITLE   =0,//タイトル
        S_READY   =1,//準備
        S_PLAY    =2,//プレイ
        S_GAMEOVER=3;//ゲームオーバー
    
    //サーフェイスビュー
    private SurfaceHolder holder;//サーフェイスホルダー
    private Thread        thread;//スレッド

    //システム
    private int     scene;    //シーン(1)
    private int     init;     //初期化(1)
    private int     tick;     //時間経過
    private int     hiscore;  //ハイスコア
    private int     score;    //スコア
    private Random  rand;     //乱数  
    private boolean touchDown;//タッチダウン
    
    //描画
    private Bitmap[] image; //イメージ
    private Paint    paint; //描画オブジェクト
    private Canvas   canvas;//キャンバス
    private Rect     rect;  //矩形
    
    //宇宙船
    private int shipY;//Y座標
    private int shipV;//加速度

    //マップ
    private int[][] map; //データ(0:天,1:地)
    private int     mapY;//Y座標
    private int     mapH;//高さ
    private int     mapV;//加速度

    //隕石
    private int[] ballX;  //X座標
    private int[] ballY;  //Y座標
    private int[] ballDir;//方向
    
    //コンストラクタ
    public ActionView(Context context) {
        super(context);

        //システム
        init=S_TITLE;
        rand=new Random();

        //描画
        paint=new Paint();
        paint.setAntiAlias(true);
        rect=new Rect();
        image=new Bitmap[7];
        for (int i=0;i<7;i++) image[i]=loadImage(i);         

        //マップ
        map=new int[2][30];
        
        //隕石
        ballX=new int[3];
        ballY=new int[3];
        ballDir=new int[3];
                
        //フォーカスの指定
        setFocusable(true);     
        setFocusableInTouchMode(true); 

        //ハイスコアの読み込み
        SharedPreferences pref;
        pref=context.getSharedPreferences(
            "PreferencesEx",Activity.MODE_PRIVATE);
        hiscore=pref.getInt("hiscore",0);
        
        //サーフェイスホルダーの生成
        holder=getHolder();
        holder.addCallback(this);
        holder.setFixedSize(getWidth(),getHeight());    
    }
    
    //イメージの読み込み
    private Bitmap loadImage(int idx) {
        int resID=R.drawable.pic0;
        if (idx==1) resID=R.drawable.pic1;
        if (idx==2) resID=R.drawable.pic2;
        if (idx==3) resID=R.drawable.pic3;
        if (idx==4) resID=R.drawable.pic4;
        if (idx==5) resID=R.drawable.pic5;
        if (idx==6) resID=R.drawable.pic6;
        return BitmapFactory.decodeResource(
            getResources(),resID);
    }

    //サーフェイスの生成
    public void surfaceCreated(SurfaceHolder holder) {
        thread=new Thread(this);
        thread.start();
    }

    //サーフェイスの終了
    public void surfaceDestroyed(SurfaceHolder holder) {
        thread=null;
    }

    //サーフェイスの変更
    public void surfaceChanged(SurfaceHolder holder,
        int format,int w,int h) {
    }   
      
    //スレッドの処理
    public void run() {     
        long sleepTime=0;
        while(thread!=null) {
            //定期処理
            canvas=holder.lockCanvas();
            onTick(canvas);
            holder.unlockCanvasAndPost(canvas);
                
            //スリープ(3)
            while (System.currentTimeMillis()<sleepTime+100);
            sleepTime=System.currentTimeMillis();
            if (++tick>=999) tick=99;
        }
    }
    
    //定期処理(2)
    private void onTick(Canvas canvas) {
        int bx=(getWidth()-300)/2;
        int by=(getHeight()-300)/2;
        //シーンの初期化
        if (init>=0) {
            scene=init;
            init=-1;
            tick=0;
            
            //タイトル
            if (scene==S_TITLE ||
                scene==S_READY) {
                //システム
                score=0;
                touchDown=false;

                //宇宙船
                shipY=120;
                shipV=0;

                //マップ
                mapY=40;
                mapH=220;
                mapV=0;
                for(int i=0;i<30;i++){
                    map[0][i]=mapY;
                    map[1][i]=mapY+mapH;
                } 
                
                //隕石
                ballY[0]=0;
                ballY[1]=0;
                ballY[2]=0;                
            }

            //ゲームオーバー
            if (scene==S_GAMEOVER) {
                if (score>hiscore) {
                    hiscore=score;
                    
                    //ハイスコアの書き込み
                    SharedPreferences pref;
                    pref=getContext().getSharedPreferences(
                        "PreferencesEx",Activity.MODE_PRIVATE);
                    SharedPreferences.Editor editor=pref.edit();
                    editor.putInt("hiscore",hiscore);
                    editor.commit();
                }
            }
        }

        //タイトル時の処理
        if (scene==S_TITLE) {
            if (touchDown) init=S_READY;
        }
        //プレイ時の処理
        else if (scene==S_PLAY) {
            //宇宙船の移動
            if (touchDown) {
                if (shipV>-16) shipV-=2;
            } else {
                if (shipV<16) shipV+=2;
            }
            shipY+=shipV;

            //隕石の移動
            for (int i=0;i<3;i++) {
                if (ballY[i]==0) continue;
                //左移動
                ballX[i]-=10;
                if (ballX[i]<-16) ballY[i]=0;
                //上下移動
                if (0<=ballX[i] && ballX[i]<300) {
                    if (ballDir[i]==1) {
                        ballY[i]-=2;
                        if (ballY[i]<=map[0][ballX[i]/10]+16) ballDir[i]=2;
                    } else if (ballDir[i]==2) {
                        ballY[i]+=2;
                        if (ballY[i]>=map[1][ballX[i]/10]-16) ballDir[i]=1;
                    }
                }                    
                //衝突判定
                if (75-16<ballX[i] && ballX[i]<75+16 &&
                    shipY-16<ballY[i] && ballY[i]<shipY+16) {
                    init=S_GAMEOVER;
                }
            }
            
            //隕石の出現
            if (mapH<=200 && tick%16==0) {
                for (int i=0;i<3;i++) {
                    if (ballY[i]!=0) continue;
                    ballX[i]=300+16;
                    ballY[i]=rand(mapH-16)+mapY+8;
                    ballDir[i]=0;
                    if ((120<=mapH && mapH<=160) || mapH<=80) {
                         ballDir[i]=1+i%2;
                    }
                    break;
                }
            }            

            //スコア加算
            score+=10;

            //マップの移動
            if (tick%20==0 && mapH>80) mapH-=4;
            if (rand(10)==0) {
                mapV=rand(20)-10;
            }
            mapY+=mapV;
            if (mapY<22) {mapY=22;mapV=Math.abs(mapV);}
            if (mapY>278-mapH) {mapY=278-mapH;mapV=-Math.abs(mapV);}
            System.arraycopy(map[0],1,map[0],0,29);
            System.arraycopy(map[1],1,map[1],0,29);
            map[0][29]=mapY;
            map[1][29]=mapY+mapH;
        }    
        //ゲームオーバー時の処理
        else if (scene==S_GAMEOVER) {
            if (touchDown && tick>=8) init=S_READY;
        }
        
        
        //マップの描画
        paint.setColor(Color.argb(255,0,0,0));
        fillRect(0,0,getWidth(),getHeight());        
        for(int i=0;i<30;i++){
            for (int j=0;j<10;j++) {
                canvas.drawBitmap(image[0],bx+i*10,by+j*30,null);
            }
            int c=Math.abs((i+score/10)%16-8)*16;
            paint.setColor(Color.argb(255,c,0,0));
            fillRect(bx+i*10,by+map[0][i],10,map[1][i]-map[0][i]);
        }    

        //隕石の描画
        for (int i=0;i<3;i++) {
            if (ballY[i]==0) continue;
            canvas.drawBitmap(image[2],bx+ballX[i]-16,by+ballY[i]-16,null);
        }        
        
        //宇宙船の描画
        if (scene!=S_GAMEOVER) {
            canvas.drawBitmap(image[1],bx+75-16,by+shipY-16,null);
            if (shipY<map[0][8] || map[1][8]<shipY) init=S_GAMEOVER;
        } else if (tick<4) {
            canvas.drawBitmap(
                image[tick+3],bx+75-16,by+shipY-24,null);
            touchDown=false;
        }

        //メッセージの描画
        if (scene==S_TITLE) {
            paint.setTextSize(12);
            drawText("Touch-down:",bx+40,by+200,0xFF000000,0xFFFFFFFF);
            drawText("Touch-up:",  bx+40,by+224,0xFF000000,0xFFFFFFFF);
            drawText("The spaceship moves up.",  bx+120,by+200,0xFF000000,0xFFFFFFFF);
            drawText("The spaceship moves down.",bx+120,by+224,0xFF000000,0xFFFFFFFF);
        } else if (scene==S_READY) {
            if (tick<5) {
                drawMessage("Ready",by+150);
            } else if (tick<7) {
            } else if (tick<10) {
                drawMessage("GO!",by+150);
            } else {
                scene=S_PLAY;
            }
        } else if (scene==S_GAMEOVER && tick>8) {
            if (score>=hiscore) {
                drawMessage("New Record",by+150);
            } else {
                  drawMessage("Game Over",by+150);
            }
        }   
        
        //スコアの描画
        drawScore(by+20);
    }

    //スコアの描画
    private void drawScore(int y) {
        String str0=""+score;
        while (str0.length()<6) str0="0"+str0;
        String str1=""+hiscore;
        while (str1.length()<6) str1="0"+str1;
        String str="SCORE "+str0+"          HISCORE "+str1;
        paint.setTextSize(16);
        int x=(int)(getWidth()-paint.measureText(str))/2;
        drawText(str,x,y,0xFF000000,0xFFFFFFFF);        
    }    
    
    //メッセージの描画
    private void drawMessage(String text,int y) {
        paint.setTextSize(24);
        int x=(int)(getWidth()-paint.measureText(text))/2;
        drawText(text,x,y,0xFFFFAAAA,0xFFFF0000);
    }
    
    //太文字の描画(4)
    private void drawText(String text,
        int x,int y,int color0,int color1) {
        paint.setColor(color0);
        canvas.drawText(text,x-1,y-1,paint);        
        canvas.drawText(text,x,y-1,paint);        
        canvas.drawText(text,x+1,y-1,paint);        
        canvas.drawText(text,x-1,y,paint);        
        canvas.drawText(text,x+1,y,paint);        
        canvas.drawText(text,x-1,y+1,paint);        
        canvas.drawText(text,x,y+1,paint);        
        canvas.drawText(text,x+1,y+1,paint);        
        paint.setColor(color1);
        canvas.drawText(text,x,y,paint);        
    }
    
    //矩形の描画
    private void fillRect(int x,int y,int w,int h) {
        rect.left=x;
        rect.top=y;
        rect.right=x+w;
        rect.bottom=y+h;
        paint.setStyle(Paint.Style.FILL);
        canvas.drawRect(rect, paint);
    }
    
    //乱数の取得(5)
    private int rand(int num) {
        return (rand.nextInt()>>>1)%num;        
    }

    //キーダウンイベントの処理
    @Override 
    public boolean onKeyDown(int keyCode,KeyEvent event) {
        if (keyCode==KeyEvent.KEYCODE_DPAD_CENTER) touchDown=true;
        return super.onKeyDown(keyCode,event);
    }

    //キーアップイベントの処理
    @Override 
    public boolean onKeyUp(int keyCode,KeyEvent event) {
        if (keyCode==KeyEvent.KEYCODE_DPAD_CENTER) touchDown=false;
        return super.onKeyUp(keyCode,event);
    }
        
    //タッチイベントの処理
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        int touchAction=event.getAction();
        if (touchAction==MotionEvent.ACTION_DOWN) {
            touchDown=true;
        } else if (touchAction==MotionEvent.ACTION_UP ||
            touchAction==MotionEvent.ACTION_CANCEL) {
            touchDown=false;
        }
        return true;
    } 
}