package net.npaka.memotool;
import android.app.Activity;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

//メモ編集
public class MemoEditor extends Activity {
    private EditText edtTitle;//タイトル
    private EditText edtBody; //本文
    private int      memoID;  //メモID
    
    //アプリの初期化
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //レイアウトの生成
        LinearLayout layout=new LinearLayout(this);
        layout.setOrientation(LinearLayout.VERTICAL);
        setContentView(layout);        
        
        //パラメータの取得
        String memoTitle="";
        Bundle extras=getIntent().getExtras();
        if (extras!=null) {
            memoID=extras.getInt("id");
            memoTitle=extras.getString("title");
        }
        
        //タイトルエディットの生成
        layout.addView(makeLabel("タイトル"));
        edtTitle=makeEditor(memoTitle,
            LinearLayout.LayoutParams.WRAP_CONTENT);
        edtTitle.setLines(1);
        layout.addView(edtTitle);

        //本文エディットの生成
        layout.addView(makeLabel("本文"));
        edtBody=makeEditor("",
            LinearLayout.LayoutParams.FILL_PARENT);
        layout.addView(edtBody);       
        readMemoBody(memoID+".txt");
    }
    
    //ラベルの生成
    private TextView makeLabel(String text) {
        TextView label=new TextView(this);
        label.setText(text);
        label.setLayoutParams(
           new LinearLayout.LayoutParams(
           LinearLayout.LayoutParams.FILL_PARENT,
           LinearLayout.LayoutParams.WRAP_CONTENT));        
        return label;       
    }
    
    //エディタの生成
    private EditText makeEditor(String text,int h) {
        EditText editor=new EditText(this);
        editor.setText(text,EditText.BufferType.NORMAL);
        editor.setGravity(Gravity.LEFT|Gravity.TOP);
        editor.setHorizontallyScrolling(true);
        editor.setLayoutParams(
            new LinearLayout.LayoutParams(
            LinearLayout.LayoutParams.FILL_PARENT,h));
        return editor;
    }

    //オプションメニューの追加
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        menu.add(0,0,0,R.string.menu_delete).setIcon(
            android.R.drawable.ic_menu_delete);
        return true;
    }    

    //オプションアイテムの選択イベント処理
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId()==0) {
            //ファイルの削除(3)
            deleteFile(memoID+".txt");
            
            //パラメータの保存            
            SharedPreferences pref=getSharedPreferences(
                "PREVIOUS_RESULT",MODE_PRIVATE);
            SharedPreferences.Editor editor=pref.edit();
            editor.putString("command","delete");
            editor.putInt("id",memoID);
            editor.commit();

            //アクティビティの終了
            setResult(RESULT_OK); 
            finish();
        }
        return true;
    }

    //キーイベントの処理
    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        //BACKキー
        if (event.getAction()==KeyEvent.ACTION_DOWN) {
            if (event.getKeyCode()==KeyEvent.KEYCODE_BACK) {
                //パラメータの保存            
                SharedPreferences pref=getSharedPreferences(
                    "PREVIOUS_RESULT",MODE_PRIVATE);
                SharedPreferences.Editor editor=pref.edit();
                editor.putString("command","update");
                editor.putInt("id",memoID);
                editor.putString("title",edtTitle.getText().toString());
                editor.commit();

                //アクティビティの終了
                writeMemoBody(memoID+".txt");
                setResult(RESULT_OK); 
            }
        }
        return super.dispatchKeyEvent(event);
    }     
    
    //メモボディの読み込み
    private void readMemoBody(String fileName) {
        String text="";
        try {
            text=Util.file2str(this,fileName);
        } catch (Exception e) {
        }
        edtBody.setText(text,EditText.BufferType.NORMAL);
    }
    
    //メモボディの書き込み
    private void writeMemoBody(String fileName) {
        try {
            Util.str2file(this,
                edtBody.getText().toString(),fileName);
        } catch (Exception e) {
            Util.showDialog(this,"","書き込み失敗しました。");
        }
    }
}
