package net.npaka.memotool;
import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.View;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import java.util.ArrayList;

//メモツール
public class MemoTool extends Activity 
    implements AdapterView.OnItemClickListener {
    private final static String BR=        
        System.getProperty("line.separator");//改行

    private final static int REQUEST_EDIT=0;//編集ID
    private ArrayList<MemoInfo> memoList;//メモリスト
    private ListView            listView;//リストビュー
    
    //アプリの初期化
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //メモリストの生成
        memoList=new ArrayList<MemoInfo>();
        readMemoList();
        
        //リストビューの生成(1)
        listView=new ListView(this);
        listView.setFocusableInTouchMode(true);
        listView.setOnItemClickListener(this);              
        this.setContentView(listView);
        updateAdapter();
    }

    //リスト選択イベントの処理
    public void onItemClick(AdapterView<?> parent,
        View view,int position,long id) {
        //メモの編集
        MemoInfo info=memoList.get(position);
        Intent intent=new Intent(this,
            net.npaka.memotool.MemoEditor.class);
        intent.putExtra("id",info.id);
        intent.putExtra("title",info.title);
        startActivityForResult(intent,REQUEST_EDIT);
    }

    //アクティビティ呼び出し結果の取得(4)
    @Override
    protected void onActivityResult(int requestCode,
        int resultCode,Intent intent) {
        if (requestCode==REQUEST_EDIT && resultCode==RESULT_OK) {
            //パラメータの取得
            SharedPreferences pref=getSharedPreferences(
                "PREVIOUS_RESULT",MODE_PRIVATE);
            String command=pref.getString("command","");
            int    id     =pref.getInt("id",0);
            String title  =pref.getString("title","");

            //更新および削除
            for (int i=0;i<memoList.size();i++) {
                MemoInfo info=memoList.get(i);
                if (info.id==id) {
                    if (command.equals("update")) {
                        info.title=title;    
                    } else if (command.equals("delete")) {
                        memoList.remove(i);
                    }                    
                    break;
                }
            }            
            writeMemoList();   
            updateAdapter();
        }
    }    
    
    //オプションメニューの追加
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        menu.add(0,0,0,R.string.menu_add).setIcon(
            android.R.drawable.ic_menu_add);
        return true;
    }    

    //オプションアイテムの選択イベント処理
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId()==0) {
            //メモの追加
            MemoInfo info=new MemoInfo();
            info.id=getFreeID();
            memoList.add(info);
            updateAdapter();
            writeMemoList();
            
            //メモの編集
            Intent intent=new Intent(this,
                net.npaka.memotool.MemoEditor.class);
            intent.putExtra("id",info.id);
            intent.putExtra("title",info.title);
            startActivityForResult(intent,0);
        }
        return true;
    }
    
    //配列アダプタの更新(2)
    private void updateAdapter() {
        String[] item=new String[memoList.size()];
        for (int i=0;i<memoList.size();i++) {
            item[i]=memoList.get(i).title;
        }
        
        //配列アダプタの生成
        ArrayAdapter<String> adapter;
        adapter=new ArrayAdapter<String>(this,
            android.R.layout.simple_list_item_1,item);
        listView.setAdapter(adapter); 
    }
    
    //空IDの取得
    private int getFreeID() {
        int id=0;
        while (true) {
            int i=0;
            for (i=0;i<memoList.size();i++) {
                MemoInfo info=memoList.get(i);
                if (info.id==id)  break;
            }
            if (i==memoList.size()) return id;
            id++;
        }
    }

    //メモリストの読み込み
    private void readMemoList() {
        try {
            memoList.clear();
            String text=Util.file2str(this,"list.txt");
            String[] line=Util.parseStr(text,BR);
            for (int i=0;i<line.length;i+=2) {
                MemoInfo info=new MemoInfo();
                info.id=Integer.parseInt(line[i]);
                info.title=line[i+1];
                memoList.add(info);
            }
        } catch (Exception e) {
        }
    }
    
    //メモリストの書き込み
    private void writeMemoList() {
        StringBuffer sb=new StringBuffer();
        for (int i=0;i<memoList.size();i++) {
            MemoInfo info=memoList.get(i);
            sb.append(""+info.id);
            sb.append(BR);
            sb.append(""+info.title);
            sb.append(BR);
        }
        try {
            Util.str2file(this,sb.toString(),"list.txt");
        } catch (Exception e) {
            Util.showDialog(this,"","書き込み失敗しました。");
        }
    }
    
    //メモ情報
    class MemoInfo {
        int    id   =-1;      //ID
        String title="タイトル";//タイトル
        
        //文字列の取得
        public String toString() {
            return title;
        }
    }
}