package net.npaka.puzzlegame;
import android.content.res.Resources;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import java.util.Random;

//パズルゲーム
public class PuzzleView extends View {    
    private Random rand=new Random();//乱数
    private Bitmap image;            //イメージ
    private int[]  data=new int[25]; //ピースデータ(1)

    //コンストラクタ
    public PuzzleView(Context context) {
        super(context);
        setBackgroundColor(Color.WHITE);

        //ピースデータの初期化
        for (int i=0;i<25;i++) data[i]=i;
        loadPicture(0);

        //フォーカスの指定
        setFocusable(true);      
        setFocusableInTouchMode(true); 
    }
   
    //ピクチャーの読み込み
    public void loadPicture(int id) {
        //ピクチャーの読み込み
        Context context=getContext();
        Resources r=context.getResources();
        int resID=R.drawable.pic0;
        if (id==1) resID=R.drawable.pic1;
        if (id==2) resID=R.drawable.pic2;
        if (id==3) resID=R.drawable.pic3;
        if (id==4) resID=R.drawable.pic4;
        if (id==5) resID=R.drawable.pic5;
        if (id==6) resID=R.drawable.pic6;
        if (id==7) resID=R.drawable.pic7;
        if (id==8) resID=R.drawable.pic8;
        if (id==9) resID=R.drawable.pic9;
        if (id==10) resID=R.drawable.pic10;
        if (id==11) resID=R.drawable.pic11;        
        image=BitmapFactory.decodeResource(r,resID); 
        
        //シャッフル
        int[] DIR={
            KeyEvent.KEYCODE_DPAD_UP,
            KeyEvent.KEYCODE_DPAD_DOWN,
            KeyEvent.KEYCODE_DPAD_LEFT,
            KeyEvent.KEYCODE_DPAD_RIGHT
        };
        for (int i=99;i>=0;i--) {
            movePiece(DIR[(rand.nextInt()>>>1)%4]);
        }        
        invalidate();
    }

    //ピースの移動(2)
    private void movePiece(int key) {
        //空きピースインデックスの検索
        int freeIdx=0;
        for (;freeIdx<25;freeIdx++) {
            if (data[freeIdx]==24) break;
        }
        
        //ピースの交換
        if (key==KeyEvent.KEYCODE_DPAD_UP && freeIdx/5<4) {
            data[freeIdx]=data[freeIdx+5];
            data[freeIdx+5]=24;
        } else if (key==KeyEvent.KEYCODE_DPAD_DOWN && freeIdx/5>0) {
            data[freeIdx]=data[freeIdx-5];
            data[freeIdx-5]=24;
        } else if (key==KeyEvent.KEYCODE_DPAD_LEFT && freeIdx%5<4) {
            data[freeIdx]=data[freeIdx+1];
            data[freeIdx+1]=24;
        } else if (key==KeyEvent.KEYCODE_DPAD_RIGHT && freeIdx%5>0) {
            data[freeIdx]=data[freeIdx-1];
            data[freeIdx-1]=24;
        }
    }    

    //ピースの描画(3)
    @Override 
    protected void onDraw(Canvas canvas) {
        if (image==null) return;

        //結果
        boolean complete=true;
        for (int i=0;i<25;i++) {
            if (data[i]!=i) complete=false;
        }
        
        //背景の描画
        Paint paint=new Paint();
        paint.setAntiAlias(true);
        paint.setStyle(Paint.Style.FILL);
        paint.setColor(Color.argb(255,0,0,0));
        canvas.drawRect(
            new Rect(0,0,getWidth(),getHeight()),
            paint);
        int bx=(getWidth()-300)/2;
        int by=(getHeight()-300)/2;
        paint.setColor(Color.argb(255,100,100,100));
        canvas.drawRect(
            new Rect(bx,by,bx+300,by+300),
            paint);
        
        //ピースの描画
        for (int i=0;i<25;i++) {
            int sx=data[i]%5;
            int sy=data[i]/5;
            int dx=i%5;
            int dy=i/5;
            if (complete || data[i]!=24) {
                canvas.drawBitmap(image,
                    new Rect(60*sx,60*sy,
                        60*sx+60,60*sy+60),
                    new Rect(bx+60*dx,by+60*dy,
                        bx+60*dx+60,by+60*dy+60),
                    null);
            }
            if (!complete) {
                paint.setColor(Color.argb(255,255,255,255));
                paint.setStyle(Paint.Style.STROKE);
                canvas.drawRect(
                    new Rect(bx+60*dx,by+60*dy,
                        bx+60*dx+60,by+60*dy+60),
                    paint);
            }
        }

        //クリアメッセージの描画
        if (complete) {
            paint.setTextSize(32);
            paint.setColor(Color.argb(255,255,0,0));
            String str="Congratulations!";
            int w=(int)paint.measureText(str);
            canvas.drawText(str,bx+(300-w)/2,by+270,paint);                
        }    
    }    
    
    //タッチイベントの処理
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (event.getAction()!=MotionEvent.ACTION_DOWN) return true;
        int touchX=(int)(event.getX()-(getWidth()-300)/2)/60;
        int touchY=(int)(event.getY()-(getHeight()-300)/2)/60;
        if (touchX<0 || 5<=touchX) return true;
        if (touchY<0 || 5<=touchY) return true;
            
        //空位置
        int freeX=0;
        int freeY=0;
        for (int i=0;i<25;i++) {
            if (data[i]==24) {
                freeX=i%5;
                freeY=i/5;
                break;
            }
        }
            
        //ピースの移動        
        if (touchX==freeX) {
            if (freeY<touchY) {
                for (int i=freeY;i<touchY;i++) {
                    movePiece(KeyEvent.KEYCODE_DPAD_UP);
                }
            } else if (freeY>touchY) {
                for (int i=freeY;i>touchY;i--) {
                    movePiece(KeyEvent.KEYCODE_DPAD_DOWN);
                }
            }
        } else if (touchY==freeY) {
            if (freeX<touchX) {
                for (int i=freeX;i<touchX;i++) {
                    movePiece(KeyEvent.KEYCODE_DPAD_LEFT);
                }
            } else if (freeX>touchX) {
                for (int i=freeX;i>touchX;i--) {
                    movePiece(KeyEvent.KEYCODE_DPAD_RIGHT);
                }
            }
        }
        invalidate();
        return true;
    }  

    //キーダウンイベントの処理
    @Override 
    public boolean onKeyDown(int keyCode,KeyEvent event) {
        movePiece(keyCode);
        invalidate();
        return super.onKeyDown(keyCode,event);
    }
}