package net.npaka.broomfight;
import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import java.util.Random;

//障害物避けゲーム
public class BroomView extends SurfaceView//(2) 
    implements SurfaceHolder.Callback,Runnable {
    //シーン定数
    private final static int S_TITLE   =1;//タイトル
    private final static int S_PLAY    =2;//プレイ
    private final static int S_GAMEOVER=3;//ゲームオーバー
    
    //サーフェイスビュー
    private SurfaceHolder holder;//サーフェイスホルダー
    private Thread        thread;//スレッド

    //システム
    private int          init =S_TITLE;     //初期化
    private int          scene=S_TITLE;     //シーン
    private int          score;             //スコア
    private boolean      keyDown;           //キーダウン
    private DoJaGraphics g;                 //グラフィックス(3)
    private Bitmap[]     image=new Bitmap[3];//画像
    private Random       rand =new Random();//乱数
        
    //ゲーム
    private int   girlX;            //女の子のX座標
    private int   girlY;            //女の子のY座標
    private int   girlSpeed;        //女の子の速度
    private int[] starX=new int[10];//星Xの座標
    private int[] starY=new int[10];//星Yの座標 
    
    //コンストラクタ
    public BroomView(Context context) {
        super(context);

        //サーフェイスホルダーの生成
        holder=getHolder();
        holder.addCallback(this);
        holder.setFixedSize(getWidth(),getHeight());
        
        //グラフィックスの取得
        g=new DoJaGraphics(holder);
        

        //画像の読み込み(4)
        Resources r=context.getResources();
        image[0]=BitmapFactory.decodeResource(r,R.drawable.pic0); 
        image[1]=BitmapFactory.decodeResource(r,R.drawable.pic1); 
        image[2]=BitmapFactory.decodeResource(r,R.drawable.pic2); 
                
        //フォーカスの指定
        setFocusable(true);     
        setFocusableInTouchMode(true);
    }

    //サーフェイスの生成
    public void surfaceCreated(SurfaceHolder holder) {
        //描画領域の指定(5)
        g.setDrawArea((getWidth()-240)/2,
            (getHeight()-240)/2,240,240);
        
        //スレッドの開始
        thread=new Thread(this);
        thread.start();
    }

    //サーフェイスの終了
    public void surfaceDestroyed(SurfaceHolder holder) {
        thread=null;
    }

    //サーフェイスの変更
    public void surfaceChanged(SurfaceHolder holder,
        int format,int w,int h) {
        //描画領域の指定(5)
        g.setDrawArea((getWidth()-240)/2,
            (getHeight()-240)/2,240,240);
    }   
      
    //スレッドの処理
    public void run() {     
        while (thread!=null) {
            tick();
            try {
                Thread.sleep(100);
            } catch (Exception e) {
            }
        }
    }

    //定期処理
    private void tick() {
        int i;
    
        //初期化
        if (init>=0) {
            //タイトルの初期化
            if (init==S_TITLE) {
                score    =0;
                girlX    =40;
                girlY    =100;
                girlSpeed=0;
                for (i=0;i<10;i++) {
                    starX[i]=-20;
                    starY[i]=0;
                }
            }
    
            //共通の初期化
            keyDown=false;
            scene  =init;
            init   =-1;
        }
    
        //共通の処理
        g.lock();
        g.drawImage(image[0],0,0);
        g.drawImage(image[1],girlX-16,girlY-16);
        for (i=0;i<10;i++) g.drawImage(image[2],starX[i]-8,starY[i]-8);
        g.drawString("SCORE:"+score,0,12);
        
        //タイトルの処理
        if (scene==S_TITLE) {
            g.drawString("BroomFight",(240-11*6)/2,100);
            if (keyDown) init=S_PLAY;
        }
        //プレイの処理
        else if (scene==S_PLAY) {
            //スコア
            score++;
    
            //女の子の位置
            if (keyDown) {
                if (girlSpeed>-20) girlSpeed--;
            } else {
                if (girlSpeed<20) girlSpeed++;
            }
            girlY+=girlSpeed;
            if (girlY<0 || 200<girlY) init=S_GAMEOVER;
            
            //星の出現
            if (rand(10)==0) {
                for (i=0;i<10;i++) {
                    if (starX[i]>-20) continue;
                    starX[i]=260;  
                    starY[i]=20+rand(170);
                    break;
                }
            }
            
            //星の移動
            for (i=0;i<10;i++) {
                if (starX[i]<=-20) continue;
                starX[i]-=6;
                if ((girlX-starX[i])*(girlX-starX[i])+
                    (girlY-starY[i])*(girlY-starY[i])<360) {
                    init=S_GAMEOVER;
                }                
            }
        }
        //ゲームオーバーの処理
        else if (scene==S_GAMEOVER) {
            g.drawString("GAME OVER",(240-9*6)/2,100);
            if (keyDown) init=S_TITLE;
        }
        
        //描画の反映
        g.unlock(true);
    }
    
    //乱数の取得
    public int rand(int num) {
        return (rand.nextInt()>>>1)%num;
    }    
    
    //タッチイベントの処理(6)
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        int touchAction=event.getAction();
        if (touchAction==MotionEvent.ACTION_DOWN) {
            keyDown=true;
        } else if (touchAction==MotionEvent.ACTION_UP ||
            touchAction==MotionEvent.ACTION_CANCEL) {
            keyDown=false;
        }
        return true;
    } 
}